/*
NeoPixels.ino - Control WS2812B LED Strips with FourBoard1.1
for use on Arduino Nano with Mega328p

A modified Adafruit NeoPixel Library Example
created 5/25/2018
http://www.hillway.us/fourboard
*/

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h>
#endif

#define PIN            3
#define NUMPIXELS      300
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

const int DIAL1PIN = A0;
const int DIAL2PIN = A1;
const int DIAL3PIN = A2;
const int DIAL4PIN = A3;

const int BUTTON1PIN = 2;
const int BUTTON2PIN = 5;
const int BUTTON3PIN = 7;
const int BUTTON4PIN = 8;

const int MOSFET1PIN = 6;
const int MOSFET2PIN = 9;
const int MOSFET3PIN = 10;
const int MOSFET4PIN = 11;

const double Base255Constant = 0.003922;

int r = 0, g = 0, b = 0;
int brightness = 0;

void setup()
{

    pinMode(DIAL1PIN, INPUT);
    pinMode(DIAL2PIN, INPUT);
    pinMode(DIAL3PIN, INPUT);
    pinMode(DIAL4PIN, INPUT);
    pinMode(BUTTON1PIN, INPUT);
    pinMode(BUTTON2PIN, INPUT);
    pinMode(BUTTON3PIN, INPUT);
    pinMode(BUTTON4PIN, INPUT);
    pinMode(MOSFET1PIN, OUTPUT);
    pinMode(MOSFET2PIN, OUTPUT);
    pinMode(MOSFET3PIN, OUTPUT);
    pinMode(MOSFET4PIN, OUTPUT);

    pixels.begin();
}

void loop()
{
    int dial1 = analogRead(DIAL1PIN);
    r = map(dial1, 0, 1023, 0, 255);
    int dial2 = analogRead(DIAL2PIN);
    g = map(dial2, 0, 1023, 0, 255);
    int dial3 = analogRead(DIAL3PIN);
    b = map(dial3, 0, 1023, 0, 255);

    int dial4 = analogRead(DIAL4PIN);
    brightness = map(dial4, 0, 1023, 0, 255);

    double conversion = (double)brightness * Base255Constant;
    r = round(conversion * (double)r);
    g = round(conversion * (double)g);
    b = round(conversion * (double)b);

    for(int i = 0; i < NUMPIXELS; i++)
        pixels.setPixelColor(i, pixels.Color(r, g, b));
    pixels.show();
}
