/*
PoweredSpeakers.ino - Sound the tones of multiple loudspeakers with FourBoard1.1
for use on Arduino Nano with Mega328p

Modified Code by Jeremy Blum http://www.jeremyblum.com/2010/09/05/driving-5-speakers-simultaneously-with-an-arduino/
created 10/2/2017
http://www.hillway.us/fourboard
*/

const int DIAL1PIN = A0;
const int DIAL2PIN = A1;
const int DIAL3PIN = A2;
const int DIAL4PIN = A3;

const int BUTTON1PIN = 2;
const int BUTTON2PIN = 5;
const int BUTTON3PIN = 7;
const int BUTTON4PIN = 8;

const int MOSFET1PIN = 6;
const int MOSFET2PIN = 9;
const int MOSFET3PIN = 10;
const int MOSFET4PIN = 11;



/* Timer reload value, globally available */
unsigned int tcnt2;

/* Toggle HIGH or LOW digital write */
int toggle1 = 0;
int toggle2 = 0;
int toggle3 = 0;
int toggle4 = 0;

/* Keep track of when each note needs to be switched */
int count1 = 0;
int count2 = 0;
int count3 = 0;
int count4 = 0;

int frequency1 = 0;
int frequency2 = 0;
int frequency3 = 0;
int frequency4 = 0;

bool s1 = false;
bool s2 = false;
bool s3 = false;
bool s4 = false;

void setup()
{
    /* First disable the timer overflow interrupt*/
    TIMSK2 &= ~(1 << TOIE2);

    /* Configure timer2 in normal mode (no PWM) */
    TCCR2A &= ~((1 << WGM21) | (1 << WGM20));
    TCCR2B &= ~(1 << WGM22);

    /* Select clock source: internal I/O clock */
    ASSR &= ~(1 << AS2);

    /* Disable Compare Match A interrupt (only overflow) */
    TIMSK2 &= ~(1 << OCIE2A);

    /* Configure the prescaler to CPU clock divided by 128 */
    TCCR2B |= (1 << CS22) | (1 << CS20); // Set bits
    TCCR2B &= ~(1 << CS21);             // Clear bit

    /* We need to calculate a proper value to load the counter.
    * The following loads the value 248 into the Timer 2 counter
    * The math behind this is:
    * (Desired period) = 64us.
    * (CPU frequency) / (prescaler value) = 125000 Hz -> 8us.
    * (desired period) / 8us = 8.
    * MAX(uint8) - 8 = 248;
    */
    /* Save value globally for later reload in ISR */
    tcnt2 = 248;

    /* Finally load end enable the timer */
    TCNT2 = tcnt2;
    TIMSK2 |= (1 << TOIE2);

    //Configure I/O Pin Directions
    pinMode(MOSFET1PIN, OUTPUT);
    pinMode(MOSFET2PIN, OUTPUT);
    pinMode(MOSFET3PIN, OUTPUT);
    pinMode(MOSFET4PIN, OUTPUT);
}

/* Install the Interrupt Service Routine (ISR) for Timer2.  */
ISR(TIMER2_OVF_vect)
{
    /* Reload the timer */
    TCNT2 = tcnt2;

    if(s1)
        count1++;
    if(s2)
        count2++;
    if(s3)
        count3++;
    if(s4)
        count4++;

    if(count1 >= frequency1)
    {
        digitalWrite(MOSFET1PIN, !toggle1);
        toggle1 = ~toggle1;
        count1 = 0;
    }
    if(count2 >= frequency2)
    {
        digitalWrite(MOSFET2PIN, !toggle2);
        toggle2 = ~toggle2;
        count2 = 0;
    }
    if(count3 >= frequency3)
    {
        digitalWrite(MOSFET3PIN, !toggle3);
        toggle3 = ~toggle3;
        count3 = 0;
    }
    if(count4 >= frequency4)
    {
        digitalWrite(MOSFET4PIN, !toggle4);
        toggle4 = ~toggle4;
        count4 = 0;
    }
}

void loop()
{
    frequency1 = map(analogRead(DIAL1PIN), 0, 1023, 400, 60);
    frequency2 = map(analogRead(DIAL2PIN), 0, 1023, 400, 53);
    frequency3 = map(analogRead(DIAL3PIN), 0, 1023, 400, 47);
    frequency4 = map(analogRead(DIAL4PIN), 0, 1023, 400, 40);

    s1 = digitalRead(BUTTON1PIN);
    s2 = digitalRead(BUTTON2PIN);
    s3 = digitalRead(BUTTON3PIN);
    s4 = digitalRead(BUTTON4PIN);
}
