/*
ReelLED.ino - Animate and Power RGBW type non-addressable LED Strips with FourBoard1.1
for use on Arduino Nano with Mega328p

created 6/14/2018
This demo code is in the public domain.
http://www.hillway.us/fourboard
*/

const int DIAL1PIN = A0;
const int DIAL2PIN = A1;
const int DIAL3PIN = A2;
const int DIAL4PIN = A3;

const int BUTTON1PIN = 2;
const int BUTTON2PIN = 5;
const int BUTTON3PIN = 7;
const int BUTTON4PIN = 8;

const int MOSFET1PIN = 6;
const int MOSFET2PIN = 9;
const int MOSFET3PIN = 10;
const int MOSFET4PIN = 11;



bool mode1 = true;
bool mode2 = false;
bool mode3 = false;
bool mode4 = false;

int r;
int g;
int b;
int w;

float rInc = 1.0;
float gInc = 1.0;
float bInc = 1.0;
float wInc = 1.0;

void setup()
{
    pinMode(DIAL1PIN, INPUT);
    pinMode(DIAL2PIN, INPUT);
    pinMode(DIAL3PIN, INPUT);
    pinMode(DIAL4PIN, INPUT);
    pinMode(BUTTON1PIN, INPUT);
    pinMode(BUTTON2PIN, INPUT);
    pinMode(BUTTON3PIN, INPUT);
    pinMode(BUTTON4PIN, INPUT);
    pinMode(MOSFET1PIN, OUTPUT);
    pinMode(MOSFET2PIN, OUTPUT);
    pinMode(MOSFET3PIN, OUTPUT);
    pinMode(MOSFET4PIN, OUTPUT);
}

void loop()
{
    if(digitalRead(BUTTON1PIN))
    {
        mode1 = true;
        mode2 = mode3 = mode4 = false;
    }
    if(digitalRead(BUTTON2PIN))
    {
        mode2 = true;
        mode1 = mode3 = mode4 = false;
    }
    if(digitalRead(BUTTON3PIN))
    {
        mode3 = true;
        mode2 = mode1 = mode4 = false;
    }
    if(digitalRead(BUTTON4PIN))
    {
        mode4 = true;
        mode2 = mode3 = mode1 = false;
    }

    if(mode1) // linear PWM control
    {
        analogWrite(MOSFET1PIN, map(analogRead(DIAL1PIN), 0, 1023, 0, 255));
        analogWrite(MOSFET2PIN, map(analogRead(DIAL2PIN), 0, 1023, 0, 255));
        analogWrite(MOSFET3PIN, map(analogRead(DIAL3PIN), 0, 1023, 0, 255));
        analogWrite(MOSFET4PIN, map(analogRead(DIAL4PIN), 0, 1023, 0, 255));
    }
    if(mode2) // quadratic PWM control
    {
        r = pow(analogRead(DIAL1PIN), 2.0) / (pow(1023.0, 2.0) / 255);
        analogWrite(MOSFET1PIN, r);

        g = pow(analogRead(DIAL2PIN), 2.0) / (pow(1023.0, 2.0) / 255);
        analogWrite(MOSFET2PIN, g);

        b = pow(analogRead(DIAL3PIN), 2.0) / (pow(1023.0, 2.0) / 255);
        analogWrite(MOSFET3PIN, b);

        w = pow(analogRead(DIAL4PIN), 2.0) / (pow(1023.0, 2.0) / 255);
        analogWrite(MOSFET4PIN, w);
    }
    if(mode3) // oscillate each color at variable speed
    {
        oscillate(analogRead(DIAL1PIN), rInc, r);
        analogWrite(MOSFET1PIN, r);

        oscillate(analogRead(DIAL2PIN), gInc, g);
        analogWrite(MOSFET2PIN, g);

        oscillate(analogRead(DIAL3PIN), bInc, b);
        analogWrite(MOSFET3PIN, b);

        oscillate(analogRead(DIAL4PIN), wInc, w);
        analogWrite(MOSFET4PIN, w);

        delay(33);
    }
    if(mode4) // oscillate each color at variable random speed
    {
        oscillate(random(analogRead(DIAL1PIN)), rInc, r);
        analogWrite(MOSFET1PIN, r);

        oscillate(random(analogRead(DIAL2PIN)), gInc, g);
        analogWrite(MOSFET2PIN, g);

        oscillate(random(analogRead(DIAL3PIN)), bInc, b);
        analogWrite(MOSFET3PIN, b);

        oscillate(random(analogRead(DIAL4PIN)), wInc, w);
        analogWrite(MOSFET4PIN, w);

        delay(33);
    }
}

void oscillate(int dial, float &inc, int &light)
{
    if(inc >= 0) inc = 25.5 * dial / 1023;
    else inc = -25.5 * dial / 1023;
    light += round(inc);
    if(light > 255) light = 255;
    if(light < 0) light = 0;
    if(light >= 255 || light <= 0) inc = -inc;
}
