/*
UnipolarStepper.ino - Drive Unipolar Stepper Motors at Variable Torque with FourBoard1.1
for use on Arduino Nano with Mega328p

created 5/3/2017
This demo code is in the public domain.
http://www.hillway.us/fourboard
*/

const int DIAL1PIN = A0;
const int DIAL2PIN = A1;
const int DIAL3PIN = A2;
const int DIAL4PIN = A3;

const int BUTTON1PIN = 2;
const int BUTTON2PIN = 5;
const int BUTTON3PIN = 7;
const int BUTTON4PIN = 8;

const int MOSFET1PIN = 6;
const int MOSFET2PIN = 9;
const int MOSFET3PIN = 10;
const int MOSFET4PIN = 11;



const int STEPS_A_REVOLUTION = 200; // for a common NEMA23
const int CHOPPER_LIMIT = 200; // 255 for no limit

int phasePosition = 0;
int torqueChopper = CHOPPER_LIMIT;
unsigned long stepAt = 0;
unsigned long stepDelay = 5000;

void setRPM(int rpm)
{
    stepDelay = 60L * 1000L * 1000L / STEPS_A_REVOLUTION / rpm;
}

void step(long steps)
{
    while(steps > 0)
    {
        unsigned long time = micros();
        if(time - stepAt >= stepDelay)
        {
            stepAt = time;
            phasePosition++;
            if(phasePosition > 3) phasePosition = 0;
            stepOnce();
            steps--;
        }
    }
    while(steps < 0)
    {
        unsigned long time = micros();
        if(time - stepAt >= stepDelay)
        {
            stepAt = time;
            phasePosition--;
            if(phasePosition < 0) phasePosition = 3;
            stepOnce();
            steps++;
        }
    }
}

void stepOnce()
{
    if(phasePosition == 0)
    {
        analogWrite(MOSFET1PIN, torqueChopper);
        analogWrite(MOSFET2PIN, LOW);
        analogWrite(MOSFET3PIN, torqueChopper);
        analogWrite(MOSFET4PIN, LOW);
    }
    if(phasePosition == 1)
    {
        analogWrite(MOSFET1PIN, LOW);
        analogWrite(MOSFET2PIN, torqueChopper);
        analogWrite(MOSFET3PIN, torqueChopper);
        analogWrite(MOSFET4PIN, LOW);
    }
    if(phasePosition == 2)
    {
        analogWrite(MOSFET1PIN, LOW);
        analogWrite(MOSFET2PIN, torqueChopper);
        analogWrite(MOSFET3PIN, LOW);
        analogWrite(MOSFET4PIN, torqueChopper);
    }
    if(phasePosition == 3)
    {
        analogWrite(MOSFET1PIN, torqueChopper);
        analogWrite(MOSFET2PIN, LOW);
        analogWrite(MOSFET3PIN, LOW);
        analogWrite(MOSFET4PIN, torqueChopper);
    }
}

void setup()
{
    pinMode(DIAL1PIN, INPUT);
    pinMode(DIAL2PIN, INPUT);
    pinMode(DIAL3PIN, INPUT);
    pinMode(DIAL4PIN, INPUT);
    pinMode(BUTTON1PIN, INPUT);
    pinMode(BUTTON2PIN, INPUT);
    pinMode(BUTTON3PIN, INPUT);
    pinMode(BUTTON4PIN, INPUT);
    pinMode(MOSFET1PIN, OUTPUT);
    pinMode(MOSFET2PIN, OUTPUT);
    pinMode(MOSFET3PIN, OUTPUT);
    pinMode(MOSFET4PIN, OUTPUT);

    setRPM(60);
}

void loop()
{
    if(digitalRead(BUTTON1PIN)) //continuous revolution
    {
        while(digitalRead(BUTTON1PIN))
        {
            int dial1 = analogRead(DIAL1PIN);
            setRPM(map(dial1, 0, 1023, 6, 180));
            int dial2 = analogRead(DIAL4PIN);
            torqueChopper = map(dial2, 0, 1023, 0, CHOPPER_LIMIT);
            step(1);
        }
    }

    if(digitalRead(BUTTON2PIN)) //one revolution
    {
        int dial1 = analogRead(DIAL1PIN);
        setRPM(map(dial1, 0, 1023, 6, 180));
        int dial2 = analogRead(DIAL4PIN);
        torqueChopper = map(dial2, 0, 1023, 0, CHOPPER_LIMIT);
        step(STEPS_A_REVOLUTION);
    }

    if(digitalRead(BUTTON3PIN)) //one reverse revolution
    {
        int dial1 = analogRead(DIAL1PIN);
        setRPM(map(dial1, 0, 1023, 6, 180));
        int dial2 = analogRead(DIAL4PIN);
        torqueChopper = map(dial2, 0, 1023, 0, CHOPPER_LIMIT);
        step(-STEPS_A_REVOLUTION);
    }

    if(digitalRead(BUTTON4PIN)) //continuous reverse revolution
    {
        while(digitalRead(BUTTON4PIN))
        {
            int dial1 = analogRead(DIAL1PIN);
            setRPM(map(dial1, 0, 1023, 6, 180));
            int dial2 = analogRead(DIAL4PIN);
            torqueChopper = map(dial2, 0, 1023, 0, CHOPPER_LIMIT);
            step(-1);
        }
    }
}
